@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2013,2014 Ivan Alonso (Kaian)
- ** Copyright (C) 2013,2014 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -32,6 +32,8 @@
 
 #include "config.h"
 #include <netdb.h>
+#include <string.h>
+#include <stdbool.h>
 #include "capture.h"
 #ifdef USE_EEP
 #include "capture_eep.h"
@@ -45,15 +47,14 @@
 #include "sip.h"
 #include "rtp.h"
 #include "setting.h"
-#include "ui_manager.h"
+#include "util.h"
 
 // Capture information
 capture_config_t capture_cfg =
 { 0 };
 
-
 void
-capture_init(int limit, int rtp_capture)
+capture_init(size_t limit, bool rtp_capture)
 {
     capture_cfg.limit = limit;
     capture_cfg.rtp_capture = rtp_capture;
@@ -73,7 +74,7 @@ capture_init(int limit, int rtp_capture)
     // Initialize calls lock
     pthread_mutexattr_t attr;
     pthread_mutexattr_init(&attr);
-#if defined(PTHREAD_MUTEX_RECURSIVE) || defined(__FreeBSD__) || defined(BSD) || defined (__OpenBSD__)
+#if defined(PTHREAD_MUTEX_RECURSIVE) || defined(__FreeBSD__) || defined(BSD) || defined (__OpenBSD__) || defined(__DragonFly__)
     pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE);
 #else
     pthread_mutexattr_settype(&attr, PTHREAD_MUTEX_RECURSIVE_NP);
@@ -91,9 +92,9 @@ capture_deinit()
     // Deallocate vectors
     vector_set_destroyer(capture_cfg.sources, vector_generic_destroyer);
     vector_destroy(capture_cfg.sources);
-    vector_set_destroyer(capture_cfg.tcp_reasm, capture_packet_destroyer);
+    vector_set_destroyer(capture_cfg.tcp_reasm, packet_destroyer);
     vector_destroy(capture_cfg.tcp_reasm);
-    vector_set_destroyer(capture_cfg.ip_reasm, capture_packet_destroyer);
+    vector_set_destroyer(capture_cfg.ip_reasm, packet_destroyer);
     vector_destroy(capture_cfg.ip_reasm);
 
     // Remove capture mutex
@@ -141,9 +142,6 @@ capture_online(const char *dev, const char *outfile)
         return 3;
     }
 
-    // Get Local devices addresses
-    pcap_findalldevs(&capture_cfg.devices, errbuf);
-
     // Add this capture information as packet source
     vector_append(capture_cfg.sources, capinfo);
 
@@ -214,8 +212,6 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
 {
     // Capture info
     capture_info_t *capinfo = (capture_info_t *) info;
-    // Source and Destination Ports
-    u_short sport, dport;
     // UDP header data
     struct udphdr *udp;
     // UDP header size
@@ -233,10 +229,10 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
     // Packet payload size
     uint32_t size_payload =  size_capture - capinfo->link_hl;
     // Captured packet info
-    capture_packet_t *pkt;
+    packet_t *pkt;
 
     // Ignore packets while capture is paused
-    if (capture_is_paused())
+    if (capture_paused())
         return;
 
     // Check if we have reached capture limit
@@ -261,8 +257,8 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
         udp_off = sizeof(struct udphdr);
 
         // Set packet ports
-        sport = htons(udp->uh_sport);
-        dport = htons(udp->uh_dport);
+        pkt->src.port = htons(udp->uh_sport);
+        pkt->dst.port = htons(udp->uh_dport);
 
         // Remove UDP Header from payload
         size_payload -= udp_off;
@@ -274,8 +270,8 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
         payload = (u_char *) (udp) + udp_off;
 
         // Complete packet with Transport information
-        capture_packet_set_transport_data(pkt, sport, dport, CAPTURE_PACKET_SIP_UDP);
-        capture_packet_set_payload(pkt, payload, size_payload);
+        packet_set_type(pkt, PACKET_SIP_UDP);
+        packet_set_payload(pkt, payload, size_payload);
 
     } else if (pkt->proto == IPPROTO_TCP) {
         // Get TCP header
@@ -283,8 +279,8 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
         tcp_off = (tcp->th_off * 4);
 
         // Set packet ports
-        sport = htons(tcp->th_sport);
-        dport = htons(tcp->th_dport);
+        pkt->src.port = htons(tcp->th_sport);
+        pkt->dst.port = htons(tcp->th_dport);
 
         // Get actual payload size
         size_payload -= tcp_off;
@@ -296,8 +292,8 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
         payload = (u_char *)(tcp) + tcp_off;
 
         // Complete packet with Transport information
-        capture_packet_set_transport_data(pkt, sport, dport, CAPTURE_PACKET_SIP_TCP);
-        capture_packet_set_payload(pkt, payload, size_payload);
+        packet_set_type(pkt, PACKET_SIP_TCP);
+        packet_set_payload(pkt, payload, size_payload);
 
         // Create a structure for this captured packet
         if (!(pkt = capture_packet_reasm_tcp(pkt, tcp, payload, size_payload)))
@@ -305,15 +301,16 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
 
 #if defined(WITH_GNUTLS) || defined(WITH_OPENSSL)
         // Check if packet is TLS
-        if (capture_cfg.keyfile)
+        if (capture_cfg.keyfile) {
             tls_process_segment(pkt, tcp);
+        }
 #endif
 
         // Check if packet is WS or WSS
         capture_ws_check_packet(pkt);
     } else {
         // Not handled protocol
-        capture_packet_destroy(pkt);
+        packet_destroy(pkt);
         return;
     }
 
@@ -330,7 +327,7 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
         dump_packet(capture_cfg.pd, pkt);
         // If storage is disabled, delete frames payload
         if (capture_cfg.storage == 0) {
-            capture_packet_free_frames(pkt);
+            packet_free_frames(pkt);
         }
         // Allow Interface refresh and user input actions
         capture_unlock();
@@ -338,12 +335,12 @@ parse_packet(u_char *info, const struct pcap_pkthdr *header, const u_char *packe
     }
 
     // Not an interesting packet ...
-    capture_packet_destroy(pkt);
+    packet_destroy(pkt);
     // Allow Interface refresh and user input actions
     capture_unlock();
 }
 
-capture_packet_t *
+packet_t *
 capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *header, u_char *packet, uint32_t *size, uint32_t *caplen)
 {
     // IP header data
@@ -369,15 +366,15 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
     // Fragmentation offset
     uint16_t ip_frag_off = 0;
     //! Source Address
-    char ip_src[ADDRESSLEN];
+    address_t src = { };
     //! Destination Address
-    char ip_dst[ADDRESSLEN];
+    address_t dst = { };
     //! Common interator for vectors
     vector_iter_t it;
     //! Packet containers
-    capture_packet_t *pkt;
+    packet_t *pkt;
     //! Storage for IP frame
-    capture_frame_t *frame;
+    frame_t *frame;
     uint32_t len_data = 0;
 
     // Get IP header
@@ -402,8 +399,8 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
             ip_frag_off = (ip_frag) ? (ip_off & IP_OFFMASK) * 8 : 0;
             ip_id = ntohs(ip4->ip_id);
 
-            inet_ntop(AF_INET, &ip4->ip_src, ip_src, sizeof(ip_src));
-            inet_ntop(AF_INET, &ip4->ip_dst, ip_dst, sizeof(ip_dst));
+            inet_ntop(AF_INET, &ip4->ip_src, src.ip, sizeof(src.ip));
+            inet_ntop(AF_INET, &ip4->ip_dst, dst.ip, sizeof(dst.ip));
             break;
 #ifdef USE_IPV6
         case 6:
@@ -417,43 +414,45 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
                 ip_id = ntohl(ip6f->ip6f_ident);
             }
 
-            inet_ntop(AF_INET6, &ip6->ip6_src, ip_src, sizeof(ip_src));
-            inet_ntop(AF_INET6, &ip6->ip6_dst, ip_dst, sizeof(ip_dst));
+            inet_ntop(AF_INET6, &ip6->ip6_src, src.ip, sizeof(src.ip));
+            inet_ntop(AF_INET6, &ip6->ip6_dst, dst.ip, sizeof(dst.ip));
             break;
 #endif
         default:
             return NULL;
     }
 
+    // Fixup VSS trailer in ethernet packets
+    *caplen = capinfo->link_hl + ip_len;
+
     // Remove IP Header length from payload
-    if (*caplen > capinfo->link_hl + ip_len) {
-        *size = ip_len - ip_hl;
-    } else {
-        *size = *caplen - capinfo->link_hl - ip_hl;
-    }
+    *size = *caplen - capinfo->link_hl - ip_hl;
 
     // If no fragmentation
     if (ip_frag == 0) {
         // Just create a new packet with given network data
-        pkt = capture_packet_create(ip_ver, ip_proto, ip_src, ip_dst, ip_id);
-        capture_packet_add_frame(pkt, header, packet);
+        pkt = packet_create(ip_ver, ip_proto, src, dst, ip_id);
+        packet_add_frame(pkt, header, packet);
         return pkt;
     }
 
     // Look for another packet with same id in IP reassembly vector
     it = vector_iterator(capture_cfg.ip_reasm);
     while ((pkt = vector_iterator_next(&it))) {
-        if (!strcmp(pkt->ip_src, ip_src) && !strcmp(pkt->ip_dst, ip_dst) && pkt->ip_id == ip_id)
+        if (addressport_equals(pkt->src, src)
+                && addressport_equals(pkt->dst, dst)
+                && pkt->ip_id == ip_id) {
             break;
+        }
     }
 
     // If we already have this packet stored, append this frames to existing one
     if (pkt) {
-        capture_packet_add_frame(pkt, header, packet);
+        packet_add_frame(pkt, header, packet);
     } else {
         // Add To the possible reassembly list
-        pkt = capture_packet_create(ip_ver, ip_proto, ip_src, ip_dst, ip_id);
-        capture_packet_add_frame(pkt, header, packet);
+        pkt = packet_create(ip_ver, ip_proto, src, dst, ip_id);
+        packet_add_frame(pkt, header, packet);
         vector_append(capture_cfg.ip_reasm, pkt);
         return NULL;
     }
@@ -495,11 +494,11 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
     return NULL;
 }
 
-capture_packet_t *
-capture_packet_reasm_tcp(capture_packet_t *packet, struct tcphdr *tcp, u_char *payload, int size_payload) {
+packet_t *
+capture_packet_reasm_tcp(packet_t *packet, struct tcphdr *tcp, u_char *payload, int size_payload) {
 
     vector_iter_t it = vector_iterator(capture_cfg.tcp_reasm);
-    capture_packet_t *pkt;
+    packet_t *pkt;
     u_char *new_payload;
 
     //! Assembled
@@ -507,20 +506,21 @@ capture_packet_reasm_tcp(capture_packet_t *packet, struct tcphdr *tcp, u_char *p
         return packet;
 
     while ((pkt = vector_iterator_next(&it))) {
-        if (!strcmp(pkt->ip_src, packet->ip_src) && !strcmp(pkt->ip_dst, packet->ip_dst)
-                && pkt->sport == packet->sport && pkt->dport == packet->dport)
+        if (addressport_equals(pkt->src, packet->src) &&
+                addressport_equals(pkt->dst, packet->dst)) {
             break;
+        }
     }
 
     // If we already have this packet stored
     if (pkt) {
-        capture_frame_t *frame;
+        frame_t *frame;
         // Append this frames to the original packet
         vector_iter_t frames = vector_iterator(packet->frames);
         while ((frame = vector_iterator_next(&frames)))
-            capture_packet_add_frame(pkt, frame->header, frame->data);
+            packet_add_frame(pkt, frame->header, frame->data);
         // Destroy current packet as its frames belong to the stored packet
-        capture_packet_destroy(packet);
+        packet_destroy(packet);
     } else {
         // First time this packet has been seen
         pkt = packet;
@@ -528,37 +528,55 @@ capture_packet_reasm_tcp(capture_packet_t *packet, struct tcphdr *tcp, u_char *p
         vector_append(capture_cfg.tcp_reasm, packet);
     }
 
+    // Store firt tcp sequence
+    if (pkt->tcp_seq == 0) {
+        pkt->tcp_seq = ntohl(tcp->th_seq);
+    }
+
     // If the first frame of this packet
     if (vector_count(pkt->frames) == 1) {
         // Set initial payload
-        capture_packet_set_payload(pkt, payload, size_payload);
+        packet_set_payload(pkt, payload, size_payload);
     } else {
         // Check payload length. Dont handle too big payload packets
         if (pkt->payload_len + size_payload > MAX_CAPTURE_LEN) {
-            capture_packet_destroy(pkt);
+            packet_destroy(pkt);
             vector_remove(capture_cfg.tcp_reasm, pkt);
             return NULL;
         }
-
-        // Append payload to the existing
         new_payload = sng_malloc(pkt->payload_len + size_payload);
-        memcpy(new_payload, pkt->payload, pkt->payload_len);
-        memcpy(new_payload + pkt->payload_len, payload, size_payload);
-        capture_packet_set_payload(pkt, new_payload, pkt->payload_len + size_payload);
+        if (pkt->tcp_seq < ntohl(tcp->th_seq)) {
+            // Append payload to the existing
+            pkt->tcp_seq =  ntohl(tcp->th_seq);
+            memcpy(new_payload, pkt->payload, pkt->payload_len);
+            memcpy(new_payload + pkt->payload_len, payload, size_payload);
+        } else {
+            // Prepend payload to the existing
+            memcpy(new_payload, payload, size_payload);
+            memcpy(new_payload + size_payload, pkt->payload, pkt->payload_len);
+        }
+        packet_set_payload(pkt, new_payload, pkt->payload_len + size_payload);
         sng_free(new_payload);
+
     }
 
     // This packet is ready to be parsed
-    if (tcp->th_flags & TH_PUSH) {
+    int valid = sip_validate_packet(pkt);
+    if (valid == VALIDATE_COMPLETE_SIP) {
+        // Full SIP packet!
+        vector_remove(capture_cfg.tcp_reasm, pkt);
+        return pkt;
+    } else if (valid == VALIDATE_NOT_SIP) {
         vector_remove(capture_cfg.tcp_reasm, pkt);
         return pkt;
     }
 
+    // An incomplete SIP Packet
     return NULL;
 }
 
 int
-capture_ws_check_packet(capture_packet_t *packet)
+capture_ws_check_packet(packet_t *packet)
 {
     int ws_off = 0;
     u_char ws_fin;
@@ -593,8 +611,8 @@ capture_ws_check_packet(capture_packet_t *packet)
      */
 
     // Get payload from packet(s)
-    size_payload = capture_packet_get_payload_len(packet);
-    payload = capture_packet_get_payload(packet);
+    size_payload = packet_payloadlen(packet);
+    payload = packet_payload(packet);
 
     // Check we have payload
     if (size_payload == 0)
@@ -646,27 +664,27 @@ capture_ws_check_packet(capture_packet_t *packet)
             newpayload[i] = newpayload[i] ^ ws_mask_key[i % 4];
     }
     // Set new packet payload into the packet
-    capture_packet_set_payload(packet, newpayload, size_payload);
+    packet_set_payload(packet, newpayload, size_payload);
     // Free the new payload
     sng_free(newpayload);
 
-    if (packet->type == CAPTURE_PACKET_SIP_TLS) {
-        capture_packet_set_type(packet, CAPTURE_PACKET_SIP_WSS);
+    if (packet->type == PACKET_SIP_TLS) {
+        packet_set_type(packet, PACKET_SIP_WSS);
     } else {
-        capture_packet_set_type(packet, CAPTURE_PACKET_SIP_WS);
+        packet_set_type(packet, PACKET_SIP_WS);
     }
     return 1;
 }
 
 
 int
-capture_packet_parse(capture_packet_t *packet)
+capture_packet_parse(packet_t *packet)
 {
     // Media structure for RTP packets
     rtp_stream_t *stream;
 
     // We're only interested in packets with payload
-    if (capture_packet_get_payload_len(packet)) {
+    if (packet_payloadlen(packet)) {
         // Parse this header and payload
         if (sip_check_packet(packet)) {
             return 0;
@@ -675,7 +693,7 @@ capture_packet_parse(capture_packet_t *packet)
         // Check if this packet belongs to a RTP stream
         if ((stream = rtp_check_packet(packet))) {
             // We have an RTP packet!
-            capture_packet_set_type(packet, CAPTURE_PACKET_RTP);
+            packet_set_type(packet, PACKET_RTP);
             // Store this pacekt if capture rtp is enabled
             if (capture_cfg.rtp_capture) {
                 call_add_rtp_packet(stream_get_call(stream), packet);
@@ -778,20 +796,20 @@ capture_set_paused(int pause)
     }
 }
 
-int
-capture_is_paused()
+bool
+capture_paused()
 {
     return capture_cfg.status == CAPTURE_ONLINE_PAUSED;
 }
 
-int
-capture_get_status()
+enum capture_status
+capture_status()
 {
     return capture_cfg.status;
 }
 
 const char *
-capture_get_status_desc()
+capture_status_desc()
 {
     switch (capture_cfg.status) {
         case CAPTURE_ONLINE:
@@ -807,13 +825,17 @@ capture_get_status_desc()
 }
 
 const char*
-capture_get_infile()
+capture_input_file()
 {
     capture_info_t *capinfo;
 
     if (vector_count(capture_cfg.sources) == 1) {
         capinfo = vector_first(capture_cfg.sources);
-        return capinfo->infile;
+        if (capinfo->infile) {
+            return sng_basename(capinfo->infile);
+        } else {
+            return NULL;
+        }
     } else {
         return "Multiple files";
     }
@@ -821,7 +843,7 @@ capture_get_infile()
 }
 
 const char*
-capture_get_keyfile()
+capture_keyfile()
 {
     return capture_cfg.keyfile;
 }
@@ -833,7 +855,7 @@ capture_set_keyfile(const char *keyfile)
 }
 
 char *
-capture_last_error(cap)
+capture_last_error()
 {
     capture_info_t *capinfo;
     if (vector_count(capture_cfg.sources) == 1) {
@@ -858,138 +880,6 @@ capture_unlock()
     pthread_mutex_unlock(&capture_cfg.lock);
 }
 
-capture_packet_t *
-capture_packet_create(uint8_t ip_ver, uint8_t proto, const char *ip_src, const char *ip_dst, uint32_t id)
-{
-    // Create a new packet
-    capture_packet_t *packet;
-    packet = sng_malloc(sizeof(capture_packet_t));
-    packet->ip_version = ip_ver;
-    packet->proto = proto;
-    packet->frames = vector_create(1, 1);
-    packet->ip_id = id;
-    memcpy(packet->ip_src, ip_src, ADDRESSLEN);
-    memcpy(packet->ip_dst, ip_dst, ADDRESSLEN);
-    return packet;
-}
-
-void
-capture_packet_destroy(capture_packet_t *packet)
-{
-    capture_frame_t *frame;
-
-    // Check we have a valid packet pointer
-    if (!packet) return;
-
-    // TODO frame destroyer?
-    vector_iter_t it = vector_iterator(packet->frames);
-    while ((frame = vector_iterator_next(&it))) {
-        sng_free(frame->header);
-        sng_free(frame->data);
-    }
-
-    // Free remaining packet data
-    vector_set_destroyer(packet->frames, vector_generic_destroyer);
-    vector_destroy(packet->frames);
-    sng_free(packet->payload);
-    sng_free(packet);
-}
-
-void
-capture_packet_destroyer(void *packet)
-{
-    capture_packet_destroy((capture_packet_t*) packet);
-}
-
-void
-capture_packet_free_frames(capture_packet_t *pkt)
-{
-    capture_frame_t *frame;
-    vector_iter_t it = vector_iterator(pkt->frames);
-
-    while ((frame = vector_iterator_next(&it))) {
-        sng_free(frame->data);
-        frame->data = NULL;
-    }
-}
-
-capture_packet_t *
-capture_packet_set_transport_data(capture_packet_t *pkt, u_short sport, u_short dport, int type)
-{
-    pkt->sport = sport;
-    pkt->dport = dport;
-    pkt->type = type;
-    return pkt;
-}
-
-capture_frame_t *
-capture_packet_add_frame(capture_packet_t *pkt, const struct pcap_pkthdr *header, const u_char *packet)
-{
-    capture_frame_t *frame;
-
-    // Add frame to this packet
-    frame = sng_malloc(sizeof(capture_frame_t));
-    frame->header = sng_malloc(sizeof(struct pcap_pkthdr));
-    memcpy(frame->header, header, sizeof(struct pcap_pkthdr));
-    frame->data = sng_malloc(header->caplen);
-    memcpy(frame->data, packet, header->caplen);
-    vector_append(pkt->frames, frame);
-    return frame;
-}
-
-void
-capture_packet_set_type(capture_packet_t *packet, int type)
-{
-    packet->type = type;
-}
-
-void
-capture_packet_set_payload(capture_packet_t *packet, u_char *payload, uint32_t payload_len)
-{
-
-    // Free previous payload
-    sng_free(packet->payload);
-    packet->payload_len = 0;
-
-    // Set new payload
-    if (payload) {
-        packet->payload = sng_malloc(payload_len + 1);
-        memset(packet->payload, 0, payload_len + 1);
-        memcpy(packet->payload, payload, payload_len);
-        packet->payload_len = payload_len;
-    }
-
-}
-
-uint32_t
-capture_packet_get_payload_len(capture_packet_t *packet)
-{
-    return packet->payload_len;
-}
-
-u_char *
-capture_packet_get_payload(capture_packet_t *packet)
-{
-    return packet->payload;
-}
-
-struct timeval
-capture_packet_get_time(capture_packet_t *packet)
-{
-    capture_frame_t *first;
-    struct timeval ts = { 0 };
-
-    // Sanity check
-    if (!packet)
-        return ts;
-
-    // Return first frame timestamp
-    if ((first = vector_first(packet->frames)))
-        ts = first->header->ts;
-
-    // Return packe timestamp
-    return ts;
-}
 
 
 void
@@ -1000,8 +890,8 @@ capture_packet_time_sorter(vector_t *vector, void *item)
     int i;
 
     // TODO Implement multiframe packets
-    curts = capture_packet_get_time(item);
-    prevts = capture_packet_get_time(vector_last(vector));
+    curts = packet_time(item);
+    prevts = packet_time(vector_last(vector));
 
     // Check if the item is already sorted
     if (timeval_is_older(curts, prevts)) {
@@ -1010,7 +900,7 @@ capture_packet_time_sorter(vector_t *vector, void *item)
 
     for (i = count - 2 ; i >= 0; i--) {
         // Get previous packet
-        prevts = capture_packet_get_time(vector_item(vector, i));
+        prevts = packet_time(vector_item(vector, i));
         // Check if the item is already in a sorted position
         if (timeval_is_older(curts, prevts)) {
             vector_insert(vector, item, i + 1);
@@ -1077,13 +967,13 @@ dump_open(const char *dumpfile)
 }
 
 void
-dump_packet(pcap_dumper_t *pd, const capture_packet_t *packet)
+dump_packet(pcap_dumper_t *pd, const packet_t *packet)
 {
     if (!pd || !packet)
         return;
 
     vector_iter_t it = vector_iterator(packet->frames);
-    capture_frame_t *frame;
+    frame_t *frame;
     while ((frame = vector_iterator_next(&it))) {
         pcap_dump((u_char*) pd, frame->header, frame->data);
     }
@@ -1097,73 +987,3 @@ dump_close(pcap_dumper_t *pd)
         return;
     pcap_dump_close(pd);
 }
-
-const char *
-lookup_hostname(const char *address)
-{
-    int i;
-    int hostlen;
-    in_addr_t netaddress;
-    struct hostent *host;
-    const char *hostname;
-
-    // No lookup enabled, return address as is
-    if (!setting_enabled(SETTING_CAPTURE_LOOKUP))
-        return address;
-
-    // Check if we have already tryied resolve this address
-    for (i = 0; i < capture_cfg.dnscache.count; i++) {
-        if (!strcmp(capture_cfg.dnscache.addr[i], address)) {
-            return capture_cfg.dnscache.hostname[i];
-        }
-    }
-
-    // Convert the address to network byte order
-    if ((netaddress = inet_addr(address)) == -1)
-        return address;
-
-    // Lookup this addres
-    host = gethostbyaddr(&netaddress, sizeof(netaddress), AF_INET);
-    if (!host) {
-        hostname = address;
-    } else {
-        hostname = host->h_name;
-    }
-
-    // Max hostname length set to 16 chars
-    hostlen = strlen(hostname);
-
-    // Store this result in the dnscache
-    strcpy(capture_cfg.dnscache.addr[capture_cfg.dnscache.count], address);
-    strncpy(capture_cfg.dnscache.hostname[capture_cfg.dnscache.count], hostname, hostlen);
-    capture_cfg.dnscache.count++;
-
-    // Return the stored value
-    return capture_cfg.dnscache.hostname[capture_cfg.dnscache.count - 1];
-}
-
-int
-is_local_address_str(const char *address)
-{
-    char straddress[ADDRESSLEN], *end;
-    strcpy(straddress, address);
-    // If address comes with port, remove it
-    if ((end = strchr(straddress, ':')))
-        *end = '\0';
-    return is_local_address(inet_addr(straddress));
-}
-
-int
-is_local_address(in_addr_t address)
-{
-    pcap_if_t *device;
-    pcap_addr_t *dev_addr;
-
-    for (device = capture_cfg.devices; device; device = device->next) {
-        for (dev_addr = device->addresses; dev_addr; dev_addr = dev_addr->next)
-            if (dev_addr->addr && dev_addr->addr->sa_family == AF_INET
-                && ((struct sockaddr_in*) dev_addr->addr)->sin_addr.s_addr == address)
-                return 1;
-    }
-    return 0;
-}
