@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2013,2014 Ivan Alonso (Kaian)
- ** Copyright (C) 2013,2014 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -68,14 +68,10 @@
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
 #include <netinet/udp.h>
+#include <stdbool.h>
+#include "packet.h"
 #include "vector.h"
 
-#ifdef INET6_ADDRSTRLEN
-#define ADDRESSLEN INET6_ADDRSTRLEN + 1
-#else
-#define ADDRESSLEN 47
-#endif
-
 //! Max allowed packet assembled size
 #define MAX_CAPTURE_LEN 20480
 
@@ -105,35 +101,6 @@ enum capture_storage {
 typedef struct capture_config capture_config_t;
 //; Shorter declaration of capture_info structure
 typedef struct capture_info capture_info_t;
-//! Shorter declaration of dns_cache structure
-typedef struct dns_cache dns_cache_t;
-//! Shorter declaration of capture_packet structure
-typedef struct capture_packet capture_packet_t;
-//! Shorter declaration of capture_frame structure
-typedef struct capture_frame capture_frame_t;
-
-//! Stored packet types
-enum capture_packet_type {
-    CAPTURE_PACKET_SIP_UDP = 0,
-    CAPTURE_PACKET_SIP_TCP,
-    CAPTURE_PACKET_SIP_TLS,
-    CAPTURE_PACKET_SIP_WS,
-    CAPTURE_PACKET_SIP_WSS,
-    CAPTURE_PACKET_RTP,
-    CAPTURE_PACKET_RTCP,
-};
-
-/**
- * @brief Storage for DNS resolved ips
- *
- * Structure to store resolved addresses when capture.lookup
- * configuration option is enabled.
- */
-struct dns_cache {
-    int count;
-    char addr[ADDRESSLEN][256];
-    char hostname[16][256];
-};
 
 /**
  * @brief Capture common configuration
@@ -142,23 +109,19 @@ struct dns_cache {
  */
 struct capture_config {
     //! Capture status
-    int status;
+    enum capture_status status;
     //! Calls capture limit. 0 for disabling
-    int limit;
+    size_t limit;
     //! Also capture RTP packets
-    int rtp_capture;
+    bool rtp_capture;
     //! Where should we store captured packets
-    int storage;
+    enum capture_storage storage;
     //! Key file for TLS decrypt
     const char *keyfile;
     //! The compiled filter expression
     struct bpf_program fp;
     //! libpcap dump file handler
     pcap_dumper_t *pd;
-    //! Cache for DNS lookups
-    dns_cache_t dnscache;
-    //! Local devices pointer
-    pcap_if_t *devices;
     //! Capture sources
     vector_t *sources;
     //! Packets pending IP reassembly
@@ -192,47 +155,6 @@ struct capture_info
     pthread_t capture_t;
 };
 
-/**
- * Packet capture data.
- *
- * One packet can contain more than one frame after
- * assembly. We assume than one SIP message has one packet
- * (maybe in multiple frames) and that one packet can only contain
- *  one SIP message.
- *
- */
-struct capture_packet {
-    // IP protocol
-    uint8_t ip_version;
-    // Transport protocol
-    uint8_t proto;
-    // Packet type as defined in capture_packet_type
-    int type;
-    // Packet source and destination address
-    char ip_src[ADDRESSLEN], ip_dst[ADDRESSLEN];
-    // Packet source and destination port
-    u_short sport, dport;
-    //! Packet IP id
-    uint16_t ip_id;
-    //! PCAP Packet payload when it can not be get from data
-    u_char *payload;
-    //! Payload length
-    uint32_t payload_len;
-    //! Packet frame list (capture_frame_t)
-    vector_t *frames;
-};
-
-/**
- *  Capture frame.
- *  One packet can contain multiple frames.
- */
-struct capture_frame {
-    //! PCAP Frame Header data
-    struct pcap_pkthdr *header;
-    //! PCAP Frame content
-    u_char *data;
-};
-
 /**
  * @brief Initialize capture data
  *
@@ -240,7 +162,7 @@ struct capture_frame {
  * @param rtp_catpure Enable rtp capture
  */
 void
-capture_init(int limit, int rtp_capture);
+capture_init(size_t limit, bool rtp_capture);
 
 /**
  * @brief Deinitialize capture data
@@ -308,7 +230,7 @@ parse_packet(u_char *capinfo, const struct pcap_pkthdr *header, const u_char *pa
  * @return a Packet structure when packet is not fragmented or fully reassembled
  * @return NULL when packet has not been completely assembled
  */
-capture_packet_t *
+packet_t *
 capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *header,
                         u_char *packet, uint32_t *size, uint32_t *caplen);
 
@@ -328,8 +250,8 @@ capture_packet_reasm_ip(capture_info_t *capinfo, const struct pcap_pkthdr *heade
  * @return a Packet structure when packet is not segmented or fully reassembled
  * @return NULL when packet has not been completely assembled
  */
-capture_packet_t *
-capture_packet_reasm_tcp(capture_packet_t *packet, struct tcphdr *tcp,
+packet_t *
+capture_packet_reasm_tcp(packet_t *packet, struct tcphdr *tcp,
                          u_char *payload, int size_payload);
 
 /**
@@ -342,7 +264,7 @@ capture_packet_reasm_tcp(capture_packet_t *packet, struct tcphdr *tcp,
  * @return 0 if packet is websocket, 1 otherwise
  */
 int
-capture_ws_check_packet(capture_packet_t *packet);
+capture_ws_check_packet(packet_t *packet);
 
 /**
  * @brief Check if the given packet structure is SIP/RTP/..
@@ -353,7 +275,7 @@ capture_ws_check_packet(capture_packet_t *packet);
  * @return 1 otherwise
  */
 int
-capture_packet_parse(capture_packet_t *pkt);
+capture_packet_parse(packet_t *pkt);
 
 /**
  * @brief Create a capture thread for online mode
@@ -402,20 +324,20 @@ capture_set_paused(int pause);
  *
  * @return 1 if capture is paused, 0 otherwise
  */
-int
-capture_is_paused();
+bool
+capture_paused();
 
 /**
  * @brief Get capture status value
  */
-int
-capture_get_status();
+enum capture_status
+capture_status();
 
 /**
  * @brief Return a string representing current capture status
  */
 const char *
-capture_get_status_desc();
+capture_status_desc();
 
 /**
  * @brief Get Input file from Offline mode
@@ -424,7 +346,7 @@ capture_get_status_desc();
  * @return NULL in Online mode
  */
 const char*
-capture_get_infile();
+capture_input_file();
 
 /**
  * @brief Get Key file from decrypting TLS packets
@@ -432,7 +354,7 @@ capture_get_infile();
  * @return given keyfile
  */
 const char*
-capture_get_keyfile();
+capture_keyfile();
 
 /**
  * @brief Set Keyfile to decrypt TLS packets
@@ -460,73 +382,6 @@ capture_lock();
 void
 capture_unlock();
 
-/**
- * @brief Allocate memory to store new packet data
- */
-capture_packet_t *
-capture_packet_create(uint8_t ip_ver, uint8_t proto, const char *ip_src, const char *ip_dst, uint32_t id);
-
-/**
- * @brief Set Transport layer information
- */
-capture_packet_t *
-capture_packet_set_transport_data(capture_packet_t *pkt, u_short sport, u_short dport, int type);
-/**
- * @brief Add a new frame to the given packet
- */
-capture_frame_t *
-capture_packet_add_frame(capture_packet_t *pkt, const struct pcap_pkthdr *header, const u_char *packet);
-
-/**
- * @brief Deallocate a packet structure memory
- */
-void
-capture_packet_destroy(capture_packet_t *packet);
-
-/**
- * @brief Destroyer function for packet vectors
- */
-void
-capture_packet_destroyer(void *packet);
-
-/**
- * @brief Free packet frames data.
- *
- * This can be used to avoid storing packet payload in memory or disk
- */
-void
-capture_packet_free_frames(capture_packet_t *pkt);
-
-/**
- * @brief Set packet type
- */
-void
-capture_packet_set_type(capture_packet_t *packet, int type);
-
-/**
- * @brief Set packet payload when it can not be get from packet
- */
-void
-capture_packet_set_payload(capture_packet_t *packet, u_char *payload, uint32_t payload_len);
-
-/**
- * @brief Getter for capture payload size
- */
-uint32_t
-capture_packet_get_payload_len(capture_packet_t *packet);
-
-/**
- * @brief Getter for capture payload pointer
- */
-u_char *
-capture_packet_get_payload(capture_packet_t *packet);
-
-/**
- * @brief Get The timestamp for a packet.
- */
-struct timeval
-capture_packet_get_time(capture_packet_t *packet);
-
 /**
  * @brief Sorter by time for captured packets
  */
@@ -558,7 +413,7 @@ dump_open(const char *dumpfile);
  * File must be previously opened with dump_open
  */
 void
-dump_packet(pcap_dumper_t *pd, const capture_packet_t *packet);
+dump_packet(pcap_dumper_t *pd, const packet_t *packet);
 
 /**
  * @brief Close a dump file
@@ -566,25 +421,6 @@ dump_packet(pcap_dumper_t *pd, const capture_packet_t *packet);
 void
 dump_close(pcap_dumper_t *pd);
 
-/**
- * @brief Try to get hostname from its address
- *
- * Try to get hostname from the given address, store it in
- * dnscache. If hostname can not be resolved, then store the
- * original address to avoid lookup again the same address.
- */
-const char *
-lookup_hostname(const char *address);
-
-/**
- * @brief Check if a given address belongs to a local device
- *
- * @param address IPv4 string format for address
- * @return 1 if address is local, 0 otherwise
- */
-int
-is_local_address_str(const char *address);
-
 /**
  * @brief Check if a given address belongs to a local device
  *
