@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2013,2014 Ivan Alonso (Kaian)
- ** Copyright (C) 2013,2014 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -34,7 +34,6 @@
 #include <getopt.h>
 #include "option.h"
 #include "vector.h"
-#include "ui_manager.h"
 #include "capture.h"
 #include "capture_eep.h"
 #ifdef WITH_GNUTLS
@@ -43,6 +42,7 @@
 #ifdef WITH_OPENSSL
 #include "capture_openssl.h"
 #endif
+#include "curses/ui_manager.h"
 
 /**
  * @brief Usage function
@@ -87,7 +87,7 @@ void
 version()
 {
     printf("%s - %s\n"
-           "Copyright (C) 2013-2015 Irontec S.L.\n"
+           "Copyright (C) 2013-2016 Irontec S.L.\n"
            "License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>.\n"
            "This is free software: you are free to change and redistribute it.\n"
            "There is NO WARRANTY, to the extent permitted by law.\n"
@@ -346,7 +346,7 @@ main(int argc, char* argv[])
         wait_for_input();
     } else {
         setbuf(stdout, NULL);
-        while(capture_get_status() != CAPTURE_OFFLINE) {
+        while(capture_status() != CAPTURE_OFFLINE) {
             if (!quiet)
                 printf("\rDialog count: %d", sip_calls_count());
             usleep(500 * 1000);
