@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2015 Ivan Alonso (Kaian)
- ** Copyright (C) 2015 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -65,7 +65,7 @@ rtp_encoding_t encodings[] = {
 };
 
 rtp_stream_t *
-stream_create(sdp_media_t *media, const char *dst, u_short dport, int type)
+stream_create(sdp_media_t *media, address_t dst, int type)
 {
     rtp_stream_t *stream;
 
@@ -76,17 +76,15 @@ stream_create(sdp_media_t *media, const char *dst, u_short dport, int type)
     // Initialize all fields
     stream->type = type;
     stream->media = media;
-    strcpy(stream->ip_dst, dst);
-    stream->dport = dport;
+    stream->dst = dst;
 
     return stream;
 }
 
 rtp_stream_t *
-stream_complete(rtp_stream_t *stream, const char *src, u_short sport)
+stream_complete(rtp_stream_t *stream, address_t src)
 {
-    strcpy(stream->ip_src, src);
-    stream->sport = sport;
+    stream->src = src;
     return stream;
 }
 
@@ -97,10 +95,10 @@ stream_set_format(rtp_stream_t *stream, uint32_t format)
 }
 
 void
-stream_add_packet(rtp_stream_t *stream, capture_packet_t *packet)
+stream_add_packet(rtp_stream_t *stream, packet_t *packet)
 {
     if (stream->pktcnt == 0)
-        stream->time = capture_packet_get_time(packet);
+        stream->time = packet_time(packet);
 
     stream->pktcnt++;
 }
@@ -141,7 +139,7 @@ stream_get_format(rtp_stream_t *stream)
 }
 
 const char *
-rtp_get_standard_format(u_int code)
+rtp_get_standard_format(uint32_t code)
 {
     int i;
 
@@ -155,10 +153,9 @@ rtp_get_standard_format(u_int code)
 }
 
 rtp_stream_t *
-rtp_check_packet(capture_packet_t *packet)
+rtp_check_packet(packet_t *packet)
 {
-    const char *src, *dst;
-    u_short sport, dport;
+    address_t src, dst;
     rtp_stream_t *stream;
     rtp_stream_t *reverse;
     u_char format = 0;
@@ -172,14 +169,12 @@ rtp_check_packet(capture_packet_t *packet)
     struct rtcp_blk_xr_voip blk_xr_voip;
 
     // Get packet data
-    payload = capture_packet_get_payload(packet);
-    size = capture_packet_get_payload_len(packet);
+    payload = packet_payload(packet);
+    size = packet_payloadlen(packet);
 
     // Get Addresses from packet
-    src = packet->ip_src;
-    dst = packet->ip_dst;
-    sport = packet->sport;
-    dport = packet->dport;
+    src = packet->src;
+    dst = packet->dst;
 
     // Check if we have at least RTP type
     if ((int32_t) size < 2)
@@ -190,13 +185,13 @@ rtp_check_packet(capture_packet_t *packet)
         return NULL;
 
     // RTP: even, RTCP: odd
-    if (((packet->dport % 2) == 0)) {
+    if (((packet->dst.port % 2) == 0)) {
 
         // Get RTP payload type
         format = RTP_PAYLOAD_TYPE(*(payload + 1));
 
         // Find the matching stream
-        stream = rtp_find_stream(src, sport, dst, dport, format);
+        stream = rtp_find_stream_format(src, dst, format);
 
         // Check if a valid stream has been found
         if (!stream)
@@ -205,20 +200,20 @@ rtp_check_packet(capture_packet_t *packet)
         // We have found a stream, but with different format
         if (stream_is_complete(stream) && stream->rtpinfo.fmtcode != format) {
             // Create a new stream for this new format
-            stream = stream_create(stream->media, dst, dport, CAPTURE_PACKET_RTP);
-            stream_complete(stream, src, sport);
+            stream = stream_create(stream->media, dst, PACKET_RTP);
+            stream_complete(stream, src);
             stream_set_format(stream, format);
             call_add_stream(msg_get_call(stream->media->msg), stream);
         }
 
         // First packet for this stream, set source data
         if (!(stream_is_complete(stream))) {
-            stream_complete(stream, src, sport);
+            stream_complete(stream, src);
             stream_set_format(stream, format);
             // Check if an stream in the opposite direction exists
-            if (!(reverse = rtp_find_call_stream(stream->media->msg->call, stream->ip_dst, stream->dport,  stream->ip_src,  stream->sport))) {
-                reverse = stream_create(stream->media, stream->ip_src, stream->sport, CAPTURE_PACKET_RTP);
-                stream_complete(reverse, stream->ip_dst, stream->dport);
+            if (!(reverse = rtp_find_call_stream(stream->media->msg->call, stream->dst, stream->src))) {
+                reverse = stream_create(stream->media, stream->src, PACKET_RTP);
+                stream_complete(reverse, stream->dst);
                 stream_set_format(reverse, format);
                 call_add_stream(msg_get_call(stream->media->msg), reverse);
             }
@@ -228,7 +223,7 @@ rtp_check_packet(capture_packet_t *packet)
         stream_add_packet(stream, packet);
     } else {
         // Find the matching stream
-        if ((stream = rtp_find_stream(src, sport, dst, dport, format))) {
+        if ((stream = rtp_find_stream(src, dst))) {
 
             // Parse all packet payload headers
             while ((int32_t) size > 0) {
@@ -296,7 +291,7 @@ rtp_check_packet(capture_packet_t *packet)
             }
 
             // Add packet to stream
-            stream_complete(stream, src, sport);
+            stream_complete(stream, src);
             stream_add_packet(stream, packet);
         }
     }
@@ -305,7 +300,57 @@ rtp_check_packet(capture_packet_t *packet)
 }
 
 rtp_stream_t *
-rtp_find_stream(const char *src, u_short sport, const char *dst, u_short dport, u_int format)
+rtp_find_stream_format(address_t src, address_t dst, uint32_t format)
+{
+    // Structure for RTP packet streams
+    rtp_stream_t *stream;
+    // Check if this is a RTP packet from active calls
+    sip_call_t *call;
+    // Iterator for active calls
+    vector_iter_t calls;
+    // Iterator for call streams
+    vector_iter_t streams;
+    // Candiate stream
+    rtp_stream_t *candidate = NULL;
+
+    // Get active calls (during conversation)
+    calls = sip_calls_iterator();
+    //vector_iterator_set_current(&calls, vector_iterator_count(&calls) - 1);
+
+    while ((call = vector_iterator_next(&calls))) {
+        streams = vector_iterator(call->streams);
+        vector_iterator_set_last(&streams);
+        while ((stream = vector_iterator_prev(&streams))) {
+            // Only look RTP packets
+            if (stream->type != PACKET_RTP)
+                continue;
+
+            // Stream complete, check source, dst
+            if (stream_is_complete(stream)) {
+                if (addressport_equals(stream->src, src) &&
+                    addressport_equals(stream->dst, dst)) {
+                    // Exact searched stream format
+                    if (stream->rtpinfo.fmtcode == format) {
+                        return stream;
+                    } else {
+                        // Matching addresses but different format
+                        candidate = stream;
+                    }
+                }
+            } else {
+                // Incomplete stream, if dst match is enough
+                if (addressport_equals(stream->dst, dst)) {
+                       return stream;
+                }
+            }
+        }
+    }
+
+    return candidate;
+}
+
+rtp_stream_t *
+rtp_find_stream(address_t src, address_t dst)
 {
     // Structure for RTP packet streams
     rtp_stream_t *stream;
@@ -320,7 +365,7 @@ rtp_find_stream(const char *src, u_short sport, const char *dst, u_short dport,
 
     while ((call = vector_iterator_next(&calls))) {
         // Check if this call has an RTP stream for current packet data
-        if ((stream = rtp_find_call_stream(call, src, sport, dst, dport))) {
+        if ((stream = rtp_find_call_stream(call, src, dst))) {
             return stream;
         }
     }
@@ -328,8 +373,9 @@ rtp_find_stream(const char *src, u_short sport, const char *dst, u_short dport,
     return NULL;
 }
 
+
 rtp_stream_t *
-rtp_find_call_stream(struct sip_call *call, const char *ip_src, u_short sport, const char *ip_dst, u_short dport)
+rtp_find_call_stream(struct sip_call *call, address_t src, address_t dst)
 {
     rtp_stream_t *stream;
     vector_iter_t it;
@@ -340,17 +386,23 @@ rtp_find_call_stream(struct sip_call *call, const char *ip_src, u_short sport, c
     // Look for an incomplete stream with this destination
     vector_iterator_set_last(&it);
     while ((stream = vector_iterator_prev(&it))) {
-        if (!strcmp(ip_dst, stream->ip_dst) && dport == stream->dport && !stream->pktcnt) {
-            return stream;
+        if (addressport_equals(dst, stream->dst)) {
+            if (!src.port) {
+                return stream;
+            } else {
+                if (!stream->pktcnt) {
+                    return stream;
+                }
+            }
         }
     }
 
     // Try to look for an incomplete stream with this destination
-    if (ip_src && sport) {
+    if (src.port) {
         vector_iterator_set_last(&it);
         while ((stream = vector_iterator_prev(&it))) {
-            if (!strcmp(ip_src, stream->ip_src) && sport == stream->sport &&
-                !strcmp(ip_dst, stream->ip_dst) && dport == stream->dport) {
+            if (addressport_equals(src, stream->src) &&
+                addressport_equals(dst, stream->dst)) {
                 return stream;
             }
         }
