@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2015 Ivan Alonso (Kaian)
- ** Copyright (C) 2015 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -44,7 +44,6 @@ setting_t settings[SETTING_COUNT] = {
     { SETTING_ALTKEY_HINT,        "hintkeyalt",         SETTING_FMT_ENUM,    SETTING_OFF, SETTING_ENUM_ONOFF },
     { SETTING_EXITPROMPT,         "exitprompt",         SETTING_FMT_ENUM,    SETTING_ON,  SETTING_ENUM_ONOFF },
     { SETTING_CAPTURE_LIMIT,      "capture.limit",      SETTING_FMT_NUMBER,  "20000",     NULL },
-    { SETTING_CAPTURE_LOOKUP,     "capture.lookup",     SETTING_FMT_ENUM,    SETTING_OFF, SETTING_ENUM_ONOFF },
     { SETTING_CAPTURE_DEVICE,     "capture.device",     SETTING_FMT_STRING,  "any",       NULL },
     { SETTING_CAPTURE_OUTFILE,    "capture.outfile",    SETTING_FMT_STRING,  "",          NULL },
     { SETTING_CAPTURE_KEYFILE,    "capture.keyfile",    SETTING_FMT_STRING,  "",          NULL },
@@ -53,7 +52,6 @@ setting_t settings[SETTING_COUNT] = {
     { SETTING_SIP_NOINCOMPLETE,   "sip.noincomplete",   SETTING_FMT_ENUM,    SETTING_ON,  SETTING_ENUM_ONOFF },
     { SETTING_SIP_CALLS,          "sip.calls",          SETTING_FMT_ENUM,    SETTING_OFF, SETTING_ENUM_ONOFF },
     { SETTING_SAVEPATH,           "savepath",           SETTING_FMT_STRING,  "",          NULL },
-    { SETTING_DISPLAY_HOST,       "displayhost",        SETTING_FMT_ENUM,    SETTING_OFF, SETTING_ENUM_ONOFF },
     { SETTING_DISPLAY_ALIAS,      "displayalias",       SETTING_FMT_ENUM,    SETTING_OFF, SETTING_ENUM_ONOFF },
     { SETTING_CL_SCROLLSTEP,      "cl.scrollstep",      SETTING_FMT_NUMBER,  "4",         NULL },
     { SETTING_CL_COLORATTR,       "cl.colorattr",       SETTING_FMT_ENUM,    SETTING_ON,  SETTING_ENUM_ONOFF },
@@ -66,7 +64,7 @@ setting_t settings[SETTING_COUNT] = {
     { SETTING_CF_SCROLLSTEP,      "cf.scrollstep",      SETTING_FMT_NUMBER,  "4",         NULL },
     { SETTING_CF_LOCALHIGHLIGHT,  "cf.localhighlight",  SETTING_FMT_ENUM,    SETTING_ON,  SETTING_ENUM_ONOFF },
     { SETTING_CF_SDP_INFO,        "cf.sdpinfo",         SETTING_FMT_ENUM,    SETTING_OFF, SETTING_ENUM_SDP_INFO },
-    { SETTING_CF_MEDIA,           "cf.media",           SETTING_FMT_ENUM,    SETTING_OFF, SETTING_ENUM_MEDIA },
+    { SETTING_CF_MEDIA,           "cf.media",           SETTING_FMT_ENUM,    SETTING_OFF, SETTING_ENUM_ONOFF },
     { SETTING_CF_DELTA,           "cf.deltatime",       SETTING_FMT_ENUM,    SETTING_ON,  SETTING_ENUM_ONOFF },
     { SETTING_CR_SCROLLSTEP,      "cr.scrollstep",      SETTING_FMT_NUMBER,  "10",        NULL },
     { SETTING_FILTER_METHODS,     "filter.methods",     SETTING_FMT_STRING,  "",          NULL },
