@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2013,2014 Ivan Alonso (Kaian)
- ** Copyright (C) 2013,2014 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -150,11 +150,16 @@ sip_init(int limit, int only_calls, int no_incomplete)
     // Create a vector to store calls
     calls.list = vector_create(200, 50);
     vector_set_destroyer(calls.list, call_destroyer);
+    vector_set_sorter(calls.list, sip_list_sorter);
     calls.active = vector_create(10, 10);
 
     // Create hash table for callid search
     hcreate(calls.limit);
 
+    // By default sort by call index ascending
+    calls.sort.by = SIP_ATTR_CALLINDEX;
+    calls.sort.asc = true;
+
     // Initialize payload parsing regexp
     match_flags = REG_EXTENDED | REG_ICASE | REG_NEWLINE;
     regcomp(&calls.reg_method, "^([a-zA-Z]+) sip:[^ ]+ SIP/2.0\r", match_flags & ~REG_NEWLINE);
@@ -164,6 +169,10 @@ sip_init(int limit, int only_calls, int no_incomplete)
     regcomp(&calls.reg_cseq, "^CSeq:[ ]*([0-9]+) .+\r$", match_flags);
     regcomp(&calls.reg_from, "^(From|f):[ ]*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
     regcomp(&calls.reg_to, "^(To|t):[ ]*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
+    regcomp(&calls.reg_valid, "^([A-Z]+ sip:|SIP/2.0 [0-9]{3})", match_flags & ~REG_NEWLINE);
+    regcomp(&calls.reg_cl, "^(Content-Length|l):[ ]*([0-9]+)\r$", match_flags);
+    regcomp(&calls.reg_body, "\r\n\r\n(.*)", match_flags & ~REG_NEWLINE);
+
 }
 
 void
@@ -184,6 +193,9 @@ sip_deinit()
     regfree(&calls.reg_cseq);
     regfree(&calls.reg_from);
     regfree(&calls.reg_to);
+    regfree(&calls.reg_valid);
+    regfree(&calls.reg_cl);
+    regfree(&calls.reg_body);
 }
 
 
@@ -214,38 +226,80 @@ sip_get_xcallid(const char *payload, char *xcallid)
     return xcallid;
 }
 
+int
+sip_validate_packet(packet_t *packet)
+{
+    uint32_t plen = packet_payloadlen(packet);
+    u_char payload[MAX_SIP_PAYLOAD];
+    regmatch_t pmatch[3];
+    char cl_header[10];
+    int content_len;
+    int bodylen;
+
+        // Max SIP payload allowed
+    if (plen == 0 || plen > MAX_SIP_PAYLOAD)
+        return VALIDATE_NOT_SIP;
+
+    // Get payload from packet(s)
+    memset(payload, 0, MAX_SIP_PAYLOAD);
+    memcpy(payload, packet_payload(packet), plen);
+
+    // Initialize variables
+    memset(cl_header, 0, sizeof(cl_header));
+
+    // Check if the first line follows SIP request or response format
+    if (regexec(&calls.reg_valid, (const char *) payload, 2, pmatch, 0) != 0) {
+        // Not a SIP message AT ALL
+        return VALIDATE_NOT_SIP;
+    }
+
+    // Check if we have Content Length header
+    if (regexec(&calls.reg_cl, (const char *) payload, 4, pmatch, 0) != 0) {
+        // Not a SIP message or not complete
+        return VALIDATE_PARTIAL_SIP;
+    }
+
+    strncpy(cl_header, (const char *)payload +  pmatch[2].rm_so, (int)pmatch[2].rm_eo - pmatch[2].rm_so);
+    content_len = atoi(cl_header);
+
+    // Check if we have Body separator field
+    if (regexec(&calls.reg_body, (const char *) payload, 2, pmatch, 0) != 0) {
+        // Not a SIP message or not complete
+        return VALIDATE_PARTIAL_SIP;
+    }
+
+    // Get the SIP message body length
+    bodylen = (int) pmatch[1].rm_eo - pmatch[1].rm_so;
+
+    return (content_len == bodylen) ? VALIDATE_COMPLETE_SIP : VALIDATE_PARTIAL_SIP;
+}
+
 sip_msg_t *
-sip_check_packet(capture_packet_t *packet)
+sip_check_packet(packet_t *packet)
 {
     ENTRY entry;
     sip_msg_t *msg;
     sip_call_t *call;
     char callid[1024], xcallid[1024];
-    const char *src, *dst;
-    u_short sport, dport;
-    char msg_src[ADDRESSLEN];
-    char msg_dst[ADDRESSLEN];
+    address_t src, dst;
     u_char payload[MAX_SIP_PAYLOAD];
+    bool newcall = false;
 
     // Max SIP payload allowed
     if (packet->payload_len > MAX_SIP_PAYLOAD)
         return NULL;
 
     // Get Addresses from packet
-    src = packet->ip_src;
-    dst = packet->ip_dst;
-    sport = packet->sport;
-    dport = packet->dport;
+    src = packet->src;
+    dst = packet->dst;
 
     // Initialize local variables
     memset(callid, 0, sizeof(callid));
     memset(xcallid, 0, sizeof(xcallid));
-    memset(msg_src, 0, sizeof(msg_src));
-    memset(msg_dst, 0, sizeof(msg_dst));
 
     // Get payload from packet(s)
     memset(payload, 0, MAX_SIP_PAYLOAD);
-    memcpy(payload, capture_packet_get_payload(packet), capture_packet_get_payload_len(packet));
+    memcpy(payload, packet_payload(packet), packet_payloadlen(packet));
 
     // Get the Call-ID of this message
     if (!sip_get_callid((const char*) payload, callid))
@@ -292,23 +346,11 @@ sip_check_packet(capture_packet_t *packet)
         entry.data = (void *) call;
         hsearch(entry, ENTER);
 
-        // Append this call to the call list
-        vector_append(calls.list, call);
-        call->index = vector_count(calls.list);
-    }
+        // Set call index
+        call->index = vector_count(calls.list) + 1;
 
-    // Store sorce address. Prefix too long IPv6 addresses with two dots
-    if (strlen(src) > 15) {
-        sprintf(msg_src, "..%s", src + strlen(src) - 13);
-    } else {
-        strcpy(msg_src, src);
-    }
-
-    // Store destination address. Prefix too long IPv6 addresses with two dots
-    if (strlen(dst) > 15) {
-        sprintf(msg_dst, "..%s", dst + strlen(dst) - 13);
-    } else {
-        strcpy(msg_dst, dst);
+        // Mark this as a new call
+        newcall = true;
     }
 
     // At this point we know we're handling an interesting SIP Packet
@@ -318,6 +360,10 @@ sip_check_packet(capture_packet_t *packet)
     if (call_msg_count(call) == 0) {
         // Parse SIP payload
         sip_parse_msg_payload(msg, payload);
+        // If this call has X-Call-Id, append it to the parent call
+        if (strlen(call->xcallid)) {
+            call_add_xcall(sip_find_by_callid(call->xcallid), call);
+        }
     }
 
     // Add the message to the call
@@ -340,6 +386,14 @@ sip_check_packet(capture_packet_t *packet)
         }
     }
 
+    if (newcall) {
+        // Append this call to the call list
+        vector_append(calls.list, call);
+    }
+
+    // Mark the list as changed
+    calls.changed = true;
+
     // Return the loaded message
     return msg;
 
@@ -350,6 +404,14 @@ skip_message:
 
 }
 
+bool
+sip_calls_has_changed()
+{
+    bool changed = calls.changed;
+    calls.changed = false;
+    return changed;
+}
+
 int
 sip_calls_count()
 {
@@ -368,16 +430,24 @@ sip_active_calls_iterator()
     return vector_iterator(calls.active);
 }
 
-void
-sip_calls_stats(int *total, int *displayed)
+vector_t *
+sip_calls_vector()
 {
+    return calls.list;
+}
+
+sip_stats_t
+sip_calls_stats()
+{
+    sip_stats_t stats;
     vector_iter_t it = vector_iterator(calls.list);
 
     // Total number of calls without filtering
-    *total = vector_iterator_count(&it);
+    stats.total = vector_iterator_count(&it);
     // Total number of calls after filtering
     vector_iterator_set_filter(&it, filter_check_call);
-    *displayed = vector_iterator_count(&it);
+    stats.displayed = vector_iterator_count(&it);
+    return stats;
 }
 
 sip_call_t *
@@ -398,82 +468,65 @@ sip_find_by_callid(const char *callid)
     return NULL;
 }
 
-sip_call_t *
-sip_find_by_xcallid(const char *xcallid)
-{
-    sip_call_t *cur;
-    vector_iter_t it = vector_iterator(calls.list);
-
-    // Find the call with the given X-Call-Id
-    while ((cur = vector_iterator_next(&it))) {
-        if (strlen(cur->xcallid) && !strcmp(cur->xcallid, xcallid)) {
-            return cur;
-        }
-    }
-    // None found
-    return NULL;
-}
-
-
-sip_call_t *
-call_get_xcall(sip_call_t *call)
-{
-    sip_call_t *xcall;
-    if (strlen(call->xcallid)) {
-        xcall = sip_find_by_callid(call->xcallid);
-    } else {
-        xcall = sip_find_by_xcallid(call->callid);
-    }
-    return xcall;
-}
-
 int
 sip_get_msg_reqresp(sip_msg_t *msg, const u_char *payload)
 {
     regmatch_t pmatch[3];
-    char reqresp[20];
+    char resp_str[256];
+    char reqresp[40];
+    char cseq[11];
+    const char *resp_def;
+
+    // Initialize variables
+    memset(pmatch, 0, sizeof(pmatch));
+    memset(resp_str, 0, sizeof(resp_str));
+    memset(reqresp, 0, sizeof(reqresp));
 
     // If not already parsed
     if (!msg->reqresp) {
-        // Initialize variables
-        memset(reqresp, 0, sizeof(reqresp));
 
         // Method & CSeq
         if (regexec(&calls.reg_method, (const char *)payload, 2, pmatch, 0) == 0) {
             sprintf(reqresp, "%.*s", (int)(pmatch[1].rm_eo - pmatch[1].rm_so), payload + pmatch[1].rm_so);
         }
 
+        // CSeq
+        if (regexec(&calls.reg_cseq, (char*)payload, 2, pmatch, 0) == 0) {
+            sprintf(cseq, "%.*s", (int)(pmatch[1].rm_eo - pmatch[1].rm_so), payload + pmatch[1].rm_so);
+            msg->cseq = atoi(cseq);
+        }
+
+
         // Response code
         if (regexec(&calls.reg_response, (const char *)payload, 3, pmatch, 0) == 0) {
+            sprintf(resp_str, "%.*s", (int)(pmatch[1].rm_eo - pmatch[1].rm_so), payload + pmatch[1].rm_so);
             sprintf(reqresp, "%.*s", (int)(pmatch[2].rm_eo - pmatch[2].rm_so), payload + pmatch[2].rm_so);
         }
 
         // Get Request/Response Code
         msg->reqresp = sip_method_from_str(reqresp);
+
+        // For response codes, check if the text matches the default
+        if (!msg_is_request(msg)) {
+            resp_def = sip_method_str(msg->reqresp);
+            if (!resp_def || strcmp(resp_def, resp_str)) {
+                msg->resp_str = strdup(resp_str);
+            }
+        }
     }
 
     return msg->reqresp;
 }
 
 const char *
-sip_get_response_str(sip_msg_t *msg, char *out)
+sip_get_msg_reqresp_str(sip_msg_t *msg)
 {
-    regmatch_t pmatch[3];
-    const char *payload;
-
-    // If not already parsed
-    if (msg_is_request(msg))
-        return NULL;
-
-    // Get message payload
-    payload = msg_get_payload(msg);
-
-    // Response code (full text)
-    if (regexec(&calls.reg_response, payload, 3, pmatch, 0) == 0) {
-        sprintf(out, "%.*s", (int)(pmatch[1].rm_eo - pmatch[1].rm_so), payload + pmatch[1].rm_so);
+    // Check if code has non-standard text
+    if (msg->resp_str) {
+        return msg->resp_str;
+    } else {
+        return sip_method_str(msg->reqresp);
     }
-
-    return out;
 }
 
 sip_msg_t *
@@ -489,13 +542,6 @@ int
 sip_parse_msg_payload(sip_msg_t *msg, const u_char *payload)
 {
     regmatch_t pmatch[4];
-    char cseq[11];
-
-    // CSeq
-    if (regexec(&calls.reg_cseq, (char*)payload, 2, pmatch, 0) == 0) {
-        sprintf(cseq, "%.*s", (int)(pmatch[1].rm_eo - pmatch[1].rm_so), payload + pmatch[1].rm_so);
-        msg->cseq = atoi(cseq);
-    }
 
     // From
     if (regexec(&calls.reg_from, (const char *)payload, 4, pmatch, 0) == 0) {
@@ -515,31 +561,25 @@ sip_parse_msg_payload(sip_msg_t *msg, const u_char *payload)
 void
 sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
 {
-    char address[ADDRESSLEN];
-    char media_address[ADDRESSLEN] = { };
-    char media_type[15] = { };
+    address_t dst, src = { };
+    char media_type[MEDIATYPELEN] = { };
     char media_format[30] = { };
-    int media_port;
-    u_int media_fmt_pref;
-    u_int media_fmt_code;
+    uint32_t media_fmt_pref;
+    uint32_t media_fmt_code;
     sdp_media_t *media = NULL;
     char *payload2, *tofree, *line;
     sip_call_t *call = msg_get_call(msg);
 
-    // Initialize variables
-    memset(address, 0, sizeof(address));
-
     // Parse each line of payload looking for sdp information
     tofree = payload2 = strdup((char*)payload);
     while ((line = strsep(&payload2, "\r\n")) != NULL) {
         // Check if we have a media string
         if (!strncmp(line, "m=", 2)) {
-            if (sscanf(line, "m=%s %d RTP/%*s %u", media_type, &media_port, &media_fmt_pref) == 3) {
+            if (sscanf(line, "m=%s %hu RTP/%*s %u", media_type, &dst.port, &media_fmt_pref) == 3) {
                 // Create a new media structure for this message
                 if ((media = media_create(msg))) {
                     media_set_type(media, media_type);
-                    media_set_port(media, media_port);
-                    media_set_address(media, media_address);
+                    media_set_address(media, dst);
                     media_set_prefered_format(media, media_fmt_pref);
                     msg_add_media(msg, media);
 
@@ -550,11 +590,12 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
                      */
                     // Create a new stream with this destination address:port
                     if (!call_msg_is_retrans(msg)) {
-                        if (!rtp_find_call_stream(call, 0, 0, media_address, media_port)) {
+                        if (!rtp_find_call_stream(call, src, dst)) {
                             // Create RTP stream
-                            call_add_stream(call, stream_create(media, media_address, media_port, CAPTURE_PACKET_RTP));
+                            call_add_stream(call, stream_create(media, dst, PACKET_RTP));
                             // Create early RTCP stream
-                            call_add_stream(call, stream_create(media, media_address, media_port + 1, CAPTURE_PACKET_RTCP));
+                            dst.port++;
+                            call_add_stream(call, stream_create(media, dst, PACKET_RTCP));
                         }
                     }
                 }
@@ -563,8 +604,8 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
 
         // Check if we have a connection string
         if (!strncmp(line, "c=", 2)) {
-            if (sscanf(line, "c=IN IP4 %s", media_address) && media) {
-                media_set_address(media, media_address);
+            if (sscanf(line, "c=IN IP4 %s", dst.ip) && media) {
+                media_set_address(media, dst);
             }
         }
 
@@ -577,10 +618,10 @@ sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
 
         // Check if we have attribute format RTCP port
         if (!strncmp(line, "a=rtcp:", 7)) {
-            if (media && sscanf(line, "a=rtcp:%u", &media_port)) {
+            if (media && sscanf(line, "a=rtcp:%hu", &dst.port)) {
                 // Create early RTCP stream
-                if (!rtp_find_call_stream(call, 0, 0, media_address, media_port)) {
-                    call_add_stream(call, stream_create(media, media_address, media_port, CAPTURE_PACKET_RTCP));
+                if (!rtp_find_call_stream(call, src, dst)) {
+                    call_add_stream(call, stream_create(media, dst, PACKET_RTCP));
                 }
             }
         }
@@ -684,15 +725,15 @@ sip_transport_str(int transport)
 {
     switch(transport)
     {
-        case CAPTURE_PACKET_SIP_UDP:
+        case PACKET_SIP_UDP:
             return "UDP";
-        case CAPTURE_PACKET_SIP_TCP:
+        case PACKET_SIP_TCP:
             return "TCP";
-        case CAPTURE_PACKET_SIP_TLS:
+        case PACKET_SIP_TLS:
             return "TLS";
-        case CAPTURE_PACKET_SIP_WS:
+        case PACKET_SIP_WS:
             return "WS";
-        case CAPTURE_PACKET_SIP_WSS:
+        case PACKET_SIP_WSS:
             return "WSS";
     }
     return "";
@@ -714,45 +755,64 @@ sip_get_msg_header(sip_msg_t *msg, char *out)
     return out;
 }
 
-const char *
-sip_address_format(const char *address)
+void
+sip_set_sort_options(sip_sort_t sort)
 {
-    // Return address formatted depending on active settings
-    if (setting_enabled(SETTING_DISPLAY_ALIAS)) {
-        return get_alias_value(address);
-    } else if (setting_enabled(SETTING_DISPLAY_HOST)) {
-        return lookup_hostname(address);
-    } else {
-        return address;
-    }
+    calls.sort = sort;
+    sip_sort_list();
 }
 
-const char *
-sip_address_port_format(const char *addrport)
+sip_sort_t
+sip_sort_options()
 {
-    static char aport[ADDRESSLEN + 6];
-    char address[ADDRESSLEN + 6];
-    int port;
+    return calls.sort;
+}
 
-    strncpy(aport, addrport, sizeof(aport));
-    if (sscanf(aport, "%[^:]:%d", address, &port) == 2) {
-        sprintf(aport, "%s:%d", sip_address_format(address), port);
+void
+sip_sort_list()
+{
+    // Cloning the vector automatically sorts it
+    vector_t *clone = vector_clone(calls.list);
+
+    // FIXME FIXME FIXME
+    // There should be a way to destroy the vector without calling the
+    // vector destroyer for each item...
+    vector_set_destroyer(calls.list, NULL);
+    vector_destroy(calls.list);
+
+    // The new sorted list
+    calls.list = clone;
+}
+
+void
+sip_list_sorter(vector_t *vector, void *item)
+{
+    sip_call_t *prev, *cur = (sip_call_t *)item;
+    int count = vector_count(vector);
+    int i;
+
+    // First item is alway sorted
+    if (vector_count(vector) == 1)
+        return;
+
+    prev = vector_item(vector, vector_count(vector) - 2);
+
+    // Check if the item is already sorted
+    if (call_attr_compare(cur, prev, calls.sort.by) == 0) {
+        return;
     }
 
-    return aport;
-}
-
-const char *
-sip_address_strip_port(char *addrport)
-{
-    char *colon;
-
-    if (!addrport)
-        return NULL;
-
-    // FIXME Make compatible with IPv6
-    if ((colon = strchr(addrport, ':')))
-        *colon = '\0';
-
-    return addrport;
+    for (i = count - 2 ; i >= 0; i--) {
+        // Get previous item
+        prev = vector_item(vector, i);
+        // Check if the item is already in a sorted position
+        int cmp = call_attr_compare(cur, prev, calls.sort.by);
+        if ((calls.sort.asc && cmp > 0) || (!calls.sort.asc && cmp < 0)) {
+            vector_insert(vector, item, i + 1);
+            return;
+        }
+    }
+
+    // Put this item at the begining of the vector
+    vector_insert(vector, item, 0);
 }
