@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2013,2014 Ivan Alonso (Kaian)
- ** Copyright (C) 2013,2014 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -68,10 +68,6 @@ enum sip_attr_id {
     SIP_ATTR_TIME,
     //! SIP Message Method or Response code
     SIP_ATTR_METHOD,
-    //! SDP Address
-    SIP_ATTR_SDP_ADDRESS,
-    //! SDP Port
-    SIP_ATTR_SDP_PORT,
     //! SIP Message transport
     SIP_ATTR_TRANSPORT,
     //! SIP Call message counter
