@@ -2,8 +2,8 @@
  **
  ** sngrep - SIP Messages flow viewer
  **
- ** Copyright (C) 2015 Ivan Alonso (Kaian)
- ** Copyright (C) 2015 Irontec SL. All rights reserved.
+ ** Copyright (C) 2013-2016 Ivan Alonso (Kaian)
+ ** Copyright (C) 2013-2016 Irontec SL. All rights reserved.
  **
  ** This program is free software: you can redistribute it and/or modify
  ** it under the terms of the GNU General Public License as published by
@@ -49,13 +49,16 @@ call_create(char *callid, char *xcallid)
     // Create an empty vector to store rtp packets
     if (setting_enabled(SETTING_CAPTURE_RTP)) {
         call->rtp_packets = vector_create(0, 40);
-        vector_set_destroyer(call->rtp_packets, capture_packet_destroyer);
+        vector_set_destroyer(call->rtp_packets, packet_destroyer);
     }
 
     // Create an empty vector to strore stream data
     call->streams = vector_create(0, 2);
     vector_set_destroyer(call->streams, vector_generic_destroyer);
 
+    // Create an empty vector to store x-calls
+    call->xcalls = vector_create(0, 1);
+
     // Initialize call filter status
     call->filtered = -1;
 
@@ -75,6 +78,8 @@ call_destroy(sip_call_t *call)
     vector_destroy(call->streams);
     // Remove all call rtp packets
     vector_destroy(call->rtp_packets);
+    // Remove all xcalls
+    vector_destroy(call->xcalls);
     // Deallocate call memory
     sng_free(call->callid);
     sng_free(call->xcallid);
@@ -87,6 +92,12 @@ call_destroyer(void *call)
     call_destroy((sip_call_t*)call);
 }
 
+bool
+call_has_changed(sip_call_t *call)
+{
+    return call->changed;
+}
+
 void
 call_add_message(sip_call_t *call, sip_msg_t *msg)
 {
@@ -94,18 +105,26 @@ call_add_message(sip_call_t *call, sip_msg_t *msg)
     msg->call = call;
     // Put this msg at the end of the msg list
     msg->index = vector_append(call->msgs, msg);
+    // Flag this call as changed
+    call->changed = true;
 }
 
 void
 call_add_stream(sip_call_t *call, rtp_stream_t *stream)
 {
+    // Store stream
     vector_append(call->streams, stream);
+    // Flag this call as changed
+    call->changed = true;
 }
 
 void
-call_add_rtp_packet(sip_call_t *call, capture_packet_t *packet)
+call_add_rtp_packet(sip_call_t *call, packet_t *packet)
 {
+    // Store packet
     vector_append(call->rtp_packets, packet);
+    // Flag this call as changed
+    call->changed = true;
 }
 
 int
@@ -140,8 +159,8 @@ call_msg_is_retrans(sip_msg_t *msg)
     it = vector_iterator(msg->call->msgs);
     vector_iterator_set_current(&it, vector_index(msg->call->msgs, msg));
     while ((prev = vector_iterator_prev(&it))) {
-        if (!strcmp(prev->packet->ip_src, msg->packet->ip_src) &&
-                !strcmp(prev->packet->ip_dst, msg->packet->ip_dst))
+        if (addressport_equals(prev->packet->src, msg->packet->src) &&
+                addressport_equals(prev->packet->dst, msg->packet->dst))
             break;
     }
 
@@ -167,7 +186,7 @@ call_update_state(sip_call_t *call, sip_msg_t *msg)
     // If this message is actually a call, get its current state
     if (call->state) {
         if (call->state == SIP_CALLSTATE_CALLSETUP) {
-            if (reqresp == 200) {
+            if (reqresp == SIP_METHOD_ACK && call->invitecseq == msg->cseq) {
                 // Alice and Bob are talking
                 call->state = SIP_CALLSTATE_INCALL;
                 call->cstart_msg = msg;
@@ -191,6 +210,7 @@ call_update_state(sip_call_t *call, sip_msg_t *msg)
     } else {
         // This is actually a call
         if (reqresp == SIP_METHOD_INVITE) {
+            call->invitecseq = msg->cseq;
             call->state = SIP_CALLSTATE_CALLSETUP;
         }
     }
@@ -257,3 +277,62 @@ call_state_to_str(int state)
     }
     return "";
 }
+
+int
+call_attr_compare(sip_call_t *one, sip_call_t *two, enum sip_attr_id id)
+{
+    char onevalue[256], twovalue[256];
+    int oneintvalue, twointvalue;
+    int comparetype; /* TODO 0 = string compare, 1 = int comprare */
+
+    switch (id) {
+        case SIP_ATTR_CALLINDEX:
+            oneintvalue = one->index;
+            twointvalue = two->index;
+            comparetype = 1;
+            break;
+        case SIP_ATTR_MSGCNT:
+            oneintvalue = call_msg_count(one);
+            twointvalue = call_msg_count(two);
+            comparetype = 1;
+            break;
+        default:
+            // Get attribute values
+            memset(onevalue, 0, sizeof(onevalue));
+            memset(twovalue, 0, sizeof(twovalue));
+            call_get_attribute(one, id, onevalue);
+            call_get_attribute(two, id, twovalue);
+            comparetype = 0;
+            break;
+    }
+
+    switch (comparetype) {
+        case 0:
+            if (strlen(twovalue) == 0 && strlen(onevalue) == 0)
+                return 0;
+            if (strlen(twovalue) == 0)
+                return 1;
+            if (strlen(onevalue) == 0)
+                return -1;
+            return strcmp(onevalue, twovalue);
+        case 1:
+            if (oneintvalue == twointvalue) return 0;
+            if (oneintvalue > twointvalue) return 1;
+            if (oneintvalue < twointvalue) return -1;
+            /* no break */
+        default:
+            return 0;
+    }
+}
+
+void
+call_add_xcall(sip_call_t *call, sip_call_t *xcall)
+{
+    if (!call || !xcall)
+        return;
+
+    // Mark this call as changed
+    call->changed = true;
+    // Add the xcall to the list
+    vector_append(call->xcalls, xcall);
+}
